#!/bin/sh
# SPDX-License-Identifier: MIT

error() {
	printf "%s\n" "$@" >&2
	exit 1
}

ps() {
	podman ps "$@"
}

start() {
	local name="$1"
	if [ -e "/etc/atmark/containers/$name.conf" ]; then
		podman_start "$name"
	else
		podman start "$name"
	fi
}

stop() {
	local name="$1"
	podman stop "$name"
}

logs() {
	local name="$1"
	podman logs "$name"
}

logs_merged() {
	logs "$@" 2>&1
}

logs_follow() {
	local name="$1"
	local pipe found="" abos_pipe podman_pid

	pipe=$(readlink "/proc/$$/fd/1") || exit

	for abos_pipe in "/proc/$PPID/fd/"*; do
		if [ "$(readlink "$abos_pipe")" = "$pipe" ]; then
			found=1
			break
		fi
	done
	# not piped?
	[ -n "$found" ] || exec podman logs -f "$name"

	podman logs -f "$name" &
	podman_pid=$!

	while [ "$(readlink "$abos_pipe")" = "$pipe" ]; do
		if ! [ -e "/proc/$podman_pid" ]; then
			wait "$podman_pid"
			exit
		fi
		sleep 5
	done

	# kill podman process if it still exists -- using job hopefully
	# ensures we do not kill a recycled pid
	kill -9 %1
}

mode="$1"
shift

case "$mode" in
ps|start|stop|logs|logs_merged|logs_follow)
	"$mode" "$@"
	;;
*)
	error "Usage: podman.sh [ps options|start/stop/logs/logs_merged/logs_follow CONTAINER]"
	;;
esac
